Package: ctrdata
Type: Package
Title: Retrieve and Analyze Clinical Trials Data from Public Registers
Version: 1.26.2
Imports: jsonlite, nodbi (>= 0.14.0), rvest, stringdist, tidyr, V8,
        httr2, stringi, lubridate, jqr, dplyr, zip, readr, rlang,
        htmlwidgets
URL: https://cran.r-project.org/package=ctrdata,
        https://rfhb.github.io/ctrdata/
BugReports: https://github.com/rfhb/ctrdata/issues
Description: A system for querying, retrieving and analyzing
        protocol- and results-related information on clinical trials from
        four public registers, the 'European Union Clinical Trials Register'
        ('EUCTR', <https://www.clinicaltrialsregister.eu/>), 
        'ClinicalTrials.gov' (<https://clinicaltrials.gov/> and also 
        translating queries the retired classic interface), the 
        'ISRCTN' (<https://www.isrctn.com/>) and the
        'European Union Clinical Trials Information System'
        ('CTIS', <https://euclinicaltrials.eu/>). 
        Trial information is downloaded, converted and stored in a database 
        ('PostgreSQL', 'SQLite', 'DuckDB' or 'MongoDB'; via package 'nodbi'). 
        Protocols, statistical analysis plans, informed consent sheets and other
        documents in registers associated with trials can also be downloaded. 
        Other functions implement trial concepts canonically across registers,
        identify deduplicated records, easily find and extract variables 
        (fields) of interest even from complex nested data as used by the 
        registers, merge variables and update queries. 
        The package can be used for monitoring, meta- and trend-analysis of
        the design and conduct as well as of the results of clinical trials 
        across registers.
        See overview in Herold, R. (2025) <doi:10.1017/rsm.2025.10061>.
License: MIT + file LICENSE
Suggests: devtools, knitr, rmarkdown, RSQLite, mongolite, tinytest (>=
        1.2.1), RPostgres, duckdb, httr, tibble, clipr, chromote
VignetteBuilder: knitr
NeedsCompilation: no
Encoding: UTF-8
Authors@R: c(person(given = "Ralf", family = "Herold", role = c("aut", "cre"),
        email = "ralf.herold@mailbox.org", comment = c(ORCID =
        "0000-0002-8148-6748")), 
        person(given = "Marek", family = "Kubica", role = "cph",
        comment = "node-xml2js library"),
        person(given = "Ivan", family = "Bozhanov", role = "cph",
        comment = "jstree library")
        )
Language: en-GB
Depends: R (>= 4.1)
Config/roxygen2/version: 8.0.0
Packaged: 2026-07-12 18:02:20 UTC; ralfherold
Author: Ralf Herold [aut, cre] (ORCID: <https://orcid.org/0000-0002-8148-6748>),
  Marek Kubica [cph] (node-xml2js library),
  Ivan Bozhanov [cph] (jstree library)
Maintainer: Ralf Herold <ralf.herold@mailbox.org>
Repository: CRAN
Date/Publication: 2026-07-12 18:20:07 UTC
Built: R 4.6.0; ; 2026-07-12 19:43:59 UTC; unix
