# Generated by roxygen2: do not edit by hand

export(ctrFindActiveSubstanceSynonyms)
export(ctrGenerateQueries)
export(ctrGetQueryUrl)
export(ctrLoadQueryIntoDb)
export(ctrOpenSearchPagesInBrowser)
export(ctrShowOneTrial)
export(dbFindFields)
export(dbFindIdsUniqueTrials)
export(dbGetFieldsIntoDf)
export(dbQueryHistory)
export(dfMergeVariablesRelevel)
export(dfName2Value)
export(dfTrials2Long)
export(f.assignmentType)
export(f.controlType)
export(f.externalLinks)
export(f.hasResults)
export(f.isMedIntervTrial)
export(f.isUniqueTrial)
export(f.likelyPlatformTrial)
export(f.numSites)
export(f.numTestArmsSubstances)
export(f.primaryEndpointDescription)
export(f.primaryEndpointResults)
export(f.resultsDate)
export(f.sampleSize)
export(f.sponsorType)
export(f.startDate)
export(f.statusRecruitment)
export(f.trialObjectives)
export(f.trialPhase)
export(f.trialPopulation)
export(f.trialTitle)
importFrom(V8,JS)
importFrom(V8,v8)
importFrom(dplyr,`%>%`)
importFrom(dplyr,case_match)
importFrom(dplyr,case_when)
importFrom(dplyr,coalesce)
importFrom(dplyr,full_join)
importFrom(dplyr,if_else)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,rows_update)
importFrom(dplyr,select)
importFrom(htmlwidgets,createWidget)
importFrom(htmlwidgets,saveWidget)
importFrom(htmlwidgets,shinyRenderWidget)
importFrom(htmlwidgets,shinyWidgetOutput)
importFrom(httr2,req_body_json)
importFrom(httr2,req_headers)
importFrom(httr2,req_options)
importFrom(httr2,req_perform)
importFrom(httr2,req_perform_parallel)
importFrom(httr2,req_retry)
importFrom(httr2,req_throttle)
importFrom(httr2,req_url)
importFrom(httr2,req_user_agent)
importFrom(httr2,request)
importFrom(httr2,resp_status)
importFrom(jqr,jq)
importFrom(jqr,jq_flags)
importFrom(jqr,jqr)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,stream_in)
importFrom(jsonlite,toJSON)
importFrom(jsonlite,validate)
importFrom(lubridate,as.period)
importFrom(lubridate,ddays)
importFrom(lubridate,dmonths)
importFrom(lubridate,duration)
importFrom(lubridate,dyears)
importFrom(lubridate,ymd_hms)
importFrom(nodbi,docdb_create)
importFrom(nodbi,docdb_delete)
importFrom(nodbi,docdb_list)
importFrom(nodbi,docdb_query)
importFrom(nodbi,docdb_update)
importFrom(nodbi,src_duckdb)
importFrom(nodbi,src_sqlite)
importFrom(readr,clipboard)
importFrom(readr,read_file)
importFrom(readr,write_file)
importFrom(rlang,.data)
importFrom(rlang,hash)
importFrom(rvest,html_attr)
importFrom(rvest,html_elements)
importFrom(rvest,html_text)
importFrom(rvest,read_html)
importFrom(rvest,read_html_live)
importFrom(stats,na.omit)
importFrom(stats,quantile)
importFrom(stats,setNames)
importFrom(stringdist,stringsimmatrix)
importFrom(stringi,stri_count_fixed)
importFrom(stringi,stri_detect_fixed)
importFrom(stringi,stri_detect_regex)
importFrom(stringi,stri_extract_all_charclass)
importFrom(stringi,stri_extract_all_regex)
importFrom(stringi,stri_extract_first)
importFrom(stringi,stri_replace_all_fixed)
importFrom(stringi,stri_replace_all_regex)
importFrom(stringi,stri_replace_first)
importFrom(stringi,stri_split_fixed)
importFrom(stringi,stri_split_regex)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,unite)
importFrom(tidyr,unnest)
importFrom(tools,toTitleCase)
importFrom(utils,URLdecode)
importFrom(utils,URLencode)
importFrom(utils,browseURL)
importFrom(utils,capture.output)
importFrom(utils,ls.str)
importFrom(utils,packageDescription)
importFrom(utils,packageVersion)
importFrom(utils,str)
importFrom(zip,unzip)
