#
# 2024-12-07
#
# npm install -g bower
#
# htmlwidgets::scaffoldWidget("ctrViewOneTrial", bowerPkg = "jstree")
# https://libraries.io/bower/jstree
# Ivan Bozhanov, MIT License
rm -rf ./lib/jquery/src
rm ./lib/jquery/dist/*map
rm ./lib/jquery/dist/jquery.js
rm ./lib/jquery/dist/jquery.slim.js
rm ./lib/jquery/dist/jquery.slim.min.js
rm ./lib/jquery/README.md
rm ./lib/jquery/.bower.json
rm ./lib/jquery/bower.json
rm ./lib/jquery/package.json
#
rm -rf ./lib/jstree/src
rm ./lib/jstree/*json
rm ./lib/jstree/dist/jstree.js
rm ./lib/jstree/.bower.json
